<?php
add_action( "wp_ajax_nopriv_pto_delete_expense_page", 
		"pto_delete_expense_page");
add_action( "wp_ajax_pto_delete_expense_page", 
		"pto_delete_expense_page");
function pto_delete_expense_page() {
	$task_id = isset($_POST['task_id']) ? $_POST['task_id'] : '';
	wp_delete_post($task_id, true);
	$return =  array( 
		'error' 	=> false,
		'redirect' 	=> admin_url() . 'admin.php?page=pto-expenses'
	);
	header('Content-type: application/json');
	echo json_encode($return);	
	exit();
}
// Request Authorisation
add_action( "wp_ajax_nopriv_pto_request_expense_auth", 
		"pto_request_expense_auth");
add_action( "wp_ajax_pto_request_expense_auth", 
		"pto_request_expense_auth");
function pto_request_expense_auth() {
	// Check post id
	$post_id = isset($_POST['expense_id']) ? $_POST['expense_id'] : '';
	if(empty($post_id)) {
		$return =  array( 
			'error' 	=> true,
			'message' 	=> '<div class="cqpim-alert cqpim-alert-danger alert-display">' . __('The Post ID is missing', 'cqpim-expenses') . '</div>'
		);
		header('Content-type: application/json');
		echo json_encode($return);	
		exit();			
	} else {
		$sending_user = wp_get_current_user();
		// Get users by Capability
		$all_users = get_users();
		$specific_users = array();
		foreach($all_users as $user){
			if(user_can($user->ID, 'cqpim_auth_expense')){
				$specific_users[] = $user->ID;
				$team = pto_get_team_from_userid($user);
				pto_add_team_notification($team, $sending_user->ID, $post_id, 'expense_auth');
			}
		}
		// Set the data
		update_post_meta($post_id, 'auth_requested', true); 
		$auth_requested_details  = array(
			'by' => $sending_user->display_name,
			'at' => time(),
		);
		update_post_meta($post_id, 'auth_requested_details', $auth_requested_details); 
		update_post_meta($post_id, 'authorised', '');
		update_post_meta($post_id, 'auth_details', array()); 
		// send the emails			
		foreach($specific_users as $user_to_send) {
			$user_obj = get_user_by('id', $user_to_send);
			$subject = get_option('cqpim_auth_email_subject');
			$content = get_option('cqpim_auth_email_content');
			$content = str_replace('%%NAME%%', $user_obj->display_name, $content);
			$content = str_replace('%%CURRENT_USER%%', $sending_user->display_name, $content);
			$content = str_replace('%%REQUEST_URL%%', get_edit_post_link($post_id), $content);
			$content = pto_replacement_patterns($content, $post_id, 'expense');
			pto_send_emails($user_obj->user_email, $subject, $content, '', array(), 'sales');
		}
		$return =  array( 
			'error' 	=> false,
			'message' 	=> '<div class="cqpim-alert cqpim-alert-success alert-display">' . __('The expense has been sent for authorisation.', 'cqpim-expenses') . '</div>'
		);
		header('Content-type: application/json');
		echo json_encode($return);	
		exit();				
	}
	exit();
}
// Authorise Request
add_action( "wp_ajax_nopriv_pto_process_expense_auth", 
		"pto_process_expense_auth");
add_action( "wp_ajax_pto_process_expense_auth", 
		"pto_process_expense_auth");
function pto_process_expense_auth() {
	$post_id = isset($_POST['expense_id']) ? $_POST['expense_id'] : '';
	$decision = isset($_POST['decision']) ? $_POST['decision'] : '';
	$notes = isset($_POST['notes']) ? $_POST['notes'] : '';
	if(empty($post_id)) {
		$return =  array( 
			'error' 	=> true,
			'message' 	=> '<div class="cqpim-alert cqpim-alert-danger alert-display">' . __('The Post ID is missing', 'cqpim-expenses') . '</div>'
		);
		header('Content-type: application/json');
		echo json_encode($return);	
		exit();			
	} else {
		if($decision == 2 && empty($notes)) {
			$return =  array( 
				'error' 	=> true,
				'message' 	=> '<div class="cqpim-alert cqpim-alert-danger alert-display">' . __('If you are declining this request you must add a note as to why.', 'cqpim-expenses') . '</div>'
			);
			header('Content-type: application/json');
			echo json_encode($return);	
			exit();					
		} else {
			$current_user = wp_get_current_user();
			update_post_meta($post_id, 'authorised', $decision);
			$authorised_details  = array(
				'by' => $current_user->display_name,
				'at' => time(),
				'for' => $notes,
			);
			update_post_meta($post_id, 'auth_details', $authorised_details); 
			update_post_meta($post_id, 'auth_requested', ''); 
			update_post_meta($post_id, 'auth_requested_details', array()); 
			$post_obj = get_post($post_id);
			$user = get_user_by('id', $post_obj->post_author);
			$team = pto_get_team_from_userid($user);
			$to = $user->user_email;
			$subject = get_option('cqpim_authorised_email_subject');
			$subject = str_replace('%%DECISION%%', $decision, $subject);
			$content = get_option('cqpim_authorised_email_content');
			$content = str_replace('%%NAME%%', $user->display_name, $content);
			$content = str_replace('%%CURRENT_USER%%', $current_user->display_name, $content);
			if($decision == 1) {
				$decision_text = __('Authorised', 'cqpim-expenses');
				pto_add_team_notification($team, $current_user->ID, $post_id, 'expense_approve');
			} else {
				$decision_text = __('Declined', 'cqpim-expenses');
				pto_add_team_notification($team, $current_user->ID, $post_id, 'expense_decline');
			}
			$content = str_replace('%%DECISION%%', $decision_text, $content);
			$content = str_replace('%%DECISION_MESSAGE%%', $notes, $content);
			$content = str_replace('%%REQUEST_URL%%', get_edit_post_link($post_id), $content);
			$content = pto_replacement_patterns($content, $post_id, 'expense');
			pto_send_emails($to, $subject, $content, '', array(), 'sales');
			$return =  array( 
				'error' 	=> false,
				'message' 	=> '<div class="cqpim-alert cqpim-alert-success alert-display">' . __('Update successful.', 'cqpim-expenses') . '</div>'
			);
			header('Content-type: application/json');
			echo json_encode($return);	
			exit();
		}
	}
	exit();
}